#! /usr/bin/env python

############################################################################
# amaroK ABX-Comparator 0.7
# (c) 2005 Bernhard Kausler <bernhardkausler@compuserve.de>
#
# Depends on: Python 2.3.4 (lower versions not tested)
############################################################################
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
############################################################################

from os import system
from os import popen
from threading import Timer
from random import choice

try:
    from Tkinter import *
except ImportError:
    error = '"Tkinter (the standard GUI library for python) is required for this script.\n\
    If you are using gentoo, try to reemerge python with the tcltk USE-Flag enabled."'
    system("kdialog --sorry " + error)
    sys.exit()
    
#Options...
DEBUG = True
OSD_ENABLED = True
HIDE_OSD = True
LAT = "1"  # dcop latency tweak 

def debug(message):
    # prints debug message
    if DEBUG == True:
        print "[ABX-Comparator] " + message

#############################
########## classes ##########
#############################

class Test:
    def __init__(self,id):
        # identifier for the test
        self.id = id

        self.ref  = "" # file name
        self.refName = "" # track name
        self.chal = ""
        self.chalName = ""
        self.length = "" #length of the reference
        self.start = 0
        self.stopp = 0
        self.X = self.ref
        self.timer = 0 # needed for the control of "stop"
       
    def getRefName(self):
        return self.refName
 
    def getChalName(self):
        return self.chalName

    # adds the first file in playlist as reference and the second as challenger
    def addFiles(self):

        if HIDE_OSD == True and OSD_ENABLED == True:
            system("dcop amarok player enableOSD false")

        system("dcop amarok player mute")

        # adding the reference
        system("dcop amarok playlist playByIndex 0")
        fd = popen("dcop amarok player encodedURL")
        self.ref = fd.readline()
        fd.close()
        fd = popen("dcop amarok player nowPlaying")
        self.refName = fd.readline()
        fd.close()


        # determine length of the reference (needed for the playtime chooser)
        fd = popen("dcop amarok player trackTotalTime")
        self.length = int(fd.readline())
        fd.close()
        self.stopp = self.length

        # adding the challenger
        system("dcop amarok playlist playByIndex 1")
        fd = popen("dcop amarok player encodedURL")
        self.chal = fd.readline()
        fd.close()
        fd = popen("dcop amarok player nowPlaying")
        self.chalName = fd.readline()
        fd.close()

        system("dcop amarok player stop")

        system("sleep 0.5 && dcop amarok player mute")

        if OSD_ENABLED == True:
            system("dcop amarok player enableOSD true")

        # stripping and formating the strings...
        self.ref = self.ref.rstrip("\n")
        self.ref = '"' + self.ref + '"'  # '"' needed for dcop calls

        self.chal = self.chal.rstrip("\n")
        self.chal = '"' + self.chal + '"'

        self.refName  = self.refName.rstrip("\n")
        self.chalName = self.chalName.rstrip("\n")
        
        # generate the first X...
        self.newX()

        return (self.ref, self.chal)

    def newX(self):
        self.X = choice([self.ref, self.chal])
        
    def playX(self):
        self.play(self.X)
   
    def playRef(self):
        self.play(self.ref)
     
    def playChal(self):
        self.play(self.chal)

    def stop(self):
        if self.timer != 0:
            self.timer.cancel()
        system("dcop amarok player stop")

    def resetStart(self):
        self.start = 0

    def resetStop(self):
        self.stopp = self.length

    def setStart(self, start): # "start" in %
        if self.start != "":
            self.start = int(start * float(self.length))

    def setStop(self, stop): # "stop" in %
        if self.stopp != "":
            self.stopp = int(stop * float(self.length))
        
    ## private ##
    def play(self, f):
        if self.timer != 0:
            self.timer.cancel()
        
        # play the f File
        if HIDE_OSD == True and OSD_ENABLED == True:
            system("dcop amarok player enableOSD false")
        system("dcop amarok player mute && dcop amarok playlist playMedia " + f)
        if self.start > 0:
            system("sleep " + LAT + " && dcop amarok player seek " + str(self.start) + " && dcop amarok player mute")
        else:
            system("dcop amarok player mute")
        if self.stopp < self.length:
            self.timer = Timer(self.stopp-self.start, system, ["dcop amarok player stop"])
            self.timer.start()
                    
        system("dcop amarok playlist clearPlaylist")
        if OSD_ENABLED == True:
            system("dcop amarok player enableOSD true")

class Statistics:
    def __init__(self):
        # identifier: [passed tests, done tests] 
        self.tests = {}
            
    def newTest(self, test):
        # floating point needed for statistical calculations...
        self.tests[test.id] = [0,0] 

    def XisRef(self, test):
        if(test.X == test.ref):
            self.add(test.id, True)
        else:
            self.add(test.id, False)
        
    def XisChal(self, test):
        if(test.X == test.chal):
            self.add(test.id, True)
        else:
            self.add(test.id, False)

    def getScore(self, test):
        return self.tests[test.id]

    def getGuessed(self, test):
        k = self.tests[id][0]
        n = self.tests[id][1]

        # sum up k from n, k+1 from n, ..., n from n
        result = 0
        for k in range(k, n+1):
            result = result + self.bin(k,n)*pow(0.5,k)*pow(0.5,n-k) 
        result = int(result*1000)
        result = float(result)/10
        return str(result) + " %"

    def reset(self, test):
        self.tests[test.id] = [0,0]

        
    ## private ##
    # some mathematics (binomic distribution etc.)
    def fac(self, n):
        if n == 0: return 1
        x = n
        n = n - 1
        while n > 0:
            x = x * n
            n = n - 1
        return x

    def bin(self, k, n):
        return self.fac(n)/(self.fac(k)*self.fac(n-k))
    
    def add(self, id, passed):
        # increase number of passed tests
        if passed == True:
            self.tests[id][0] = self.tests[id][0] + 1
        # increase number of done tests
        self.tests[id][1] = self.tests[id][1] + 1

#########################
########## GUI ##########
#########################

class PlaytimeChooser:
    def __init__(self, master, length, height):
        def callback(event):
            co = self.can.coords(self.box)
            pivot = (co[0]+co[2])/2

            if event.x > pivot:
                self.can.coords(self.box,co[0],co[1],event.x,co[3])
            else:
                self.can.coords(self.box,event.x,co[1],co[2],co[3])

        self.length = length

        self.can = Canvas(master, height=height, width=length, bd=1, bg="black")
        self.box = self.can.create_rectangle(0,0,length,height, fill="green", width=0)

        self.can.bind("<Button-1>", callback)

    def getStart(self):
        co = self.can.coords(self.box)
        return co[0]/self.length

    def getStop(self):
        co = self.can.coords(self.box)
        return co[2]/self.length

    def resetStart(self):
        co = self.can.coords(self.box) 
        self.can.coords(self.box,0,0,co[2],co[3])

    def resetStop(self):
        co = self.can.coords(self.box)
        self.can.coords(self.box,co[0],co[1],self.length,co[3])

class Comparator:
    def __init__(self, test, stat):
        def addFiles():
            files = test.addFiles()
            lRefName.config(text=test.getRefName())
            lChalName.config(text=test.getChalName())
        
        def makeTest():
            system("dcop amarok playlist clearPlaylist")
            bPlayX.config(state=NORMAL)
            bPlayRef.config(state=NORMAL)
            bPlayChal.config(state=NORMAL)
            bStop.config(state=NORMAL)
            rbXisRef.config(state=NORMAL)
            rbXisChal.config(state=NORMAL)
            bNextTrial.config(state=NORMAL)
            bMakeTest.config(state=DISABLED)
            bAddFiles.config(state=DISABLED)

        def playX():
            test.setStart(plc.getStart())
            test.setStop(plc.getStop())
            test.playX()
            status.set("Playing X...")
                   
        def playRef():
            test.setStart(plc.getStart())
            test.setStop(plc.getStop())
            test.playRef()
            status.set("Playing reference...")
            
        def playChal():
            test.setStart(plc.getStart())
            test.setStop(plc.getStop())
            test.playChal()
            status.set("Playing challenger...")
            
        def stop():
            test.stop()
            status.set("Stopped.")

        def resetStart():
            plc.resetStart()
            test.resetStart()

        def resetStop():
            plc.resetStop()
            test.resetStart()

        def nextTrial():
            stop()
            if choice.get() == "XisRef":
                stat.XisRef(test)
            else:
                stat.XisChal(test)
            score.set(str(stat.getScore(test)[0]) + "/" + str(stat.getScore(test)[1]))
            guessed.set(stat.getGuessed(test))
            test.newX()

        def reset():
            bPlayX.config(state=DISABLED)
            bPlayRef.config(state=DISABLED)
            bPlayChal.config(state=DISABLED)
            bStop.config(state=DISABLED)
            rbXisRef.config(state=DISABLED)
            rbXisChal.config(state=DISABLED)
            bNextTrial.config(state=DISABLED)
            bMakeTest.config(state=NORMAL)
            bAddFiles.config(state=NORMAL)

            score.set("0/0")
            guessed.set("N/A")
            stat.reset(test)

        def exit():
            stop()
            root.quit()
        
        root = Tk()
        root.title("amaroK ABX-Comparator 0.7")

        status = StringVar()
        score = StringVar()
        guessed = StringVar()
        
        ################################################
        ## The main frames and the exit/reset buttons ##
        ################################################

        fFiles    = Frame(root)
        fMakeTest = Frame(root, bd=1, relief=RAISED)
        fPlayback = Frame(root, bd=1, relief=RAISED)
        fChoice   = Frame(root, bd=1, relief=RAISED)
        fChooser  = Frame(root, bd=1, relief=RAISED)
        fStatus   = Frame(root, bd=1, relief=RAISED)

        bReset    = Button(root, text="Reset", command=reset)
        bExit     = Button(root, text="Exit", command=exit)

        fFiles.grid(row=0, column=0, padx=5, pady=5, sticky=W)
        fMakeTest.grid(row=0, column=1, padx=5, pady=5, sticky=N+W+S+E)
        fPlayback.grid(row=1, column=0, padx=5, pady=5, sticky=N+S+W+E)
        fChoice.grid(row=1, column=1, padx=5, pady=5, sticky=N+S+W+E)
        fChooser.grid(row=2, column=0, columnspan=2, padx=5, pady=5, sticky=N+S+W+E)
        fStatus.grid(row=3, column=0, columnspan=2, padx=5, pady=5, sticky=N+S+W+E)
        bReset.grid(row=4, column=0, sticky=W)
        bExit.grid(row=4, column=1, sticky=E)

        #################
        #the files frame#
        #################

        lRef      = Label(fFiles, text="Reference (A):")
        lRefName  = Label(fFiles, text="N/A")

        lChal     = Label(fFiles, text="Challenger (B):")
        lChalName = Label(fFiles, text="N/A")


        lRef.grid(row=0, column=0, sticky=W)
        lRefName.grid(row=1, column=0, sticky=W)
        lChal.grid(row=2, column=0, sticky=W)
        lChalName.grid(row=3,column=0, sticky=W)


        ########################
        # the "makeTest" frame #
        ########################

        bAddFiles = Button(fMakeTest, text="Add files from playlist", width=17, command=addFiles)
        bMakeTest = Button(fMakeTest, text="Make test!", width=17, command=makeTest)

        bAddFiles.grid(sticky=W)
        bMakeTest.grid(sticky=W)


        #####################
        # the playback frame#
        #####################
        
        bPlayRef = Button(fPlayback, text="Play A",  width=12, command=playRef, state=DISABLED)
        bPlayChal = Button(fPlayback, text="Play B", width=12, command=playChal, state=DISABLED)
        bPlayX = Button(fPlayback, text="Play X",          width=12, command=playX, state=DISABLED)
        bStop  = Button(fPlayback, text="Stop",            width=12, command=stop, state=DISABLED)
 
        bPlayRef.grid()
        bPlayChal.grid(row=0,column=1)
        bPlayX.grid(row=1, columnspan=2)
        bStop.grid(row=2, columnspan=2)

        fPlayback.columnconfigure(0, pad=5)
        fPlayback.columnconfigure(1, pad=5)
        fPlayback.rowconfigure(0, pad=5)
        fPlayback.rowconfigure(1, pad=5)
        fPlayback.rowconfigure(2, pad=5)


        ###################
        # the choice frame#
        ###################
        
        choice = StringVar()
        choice.set("XisA")
        rbXisRef = Radiobutton(fChoice, text="X is A", variable=choice, value = "XisRef", state=DISABLED)
        rbXisRef.select()
        rbXisChal = Radiobutton(fChoice, text="X is B", variable=choice, value = "XisChal", state=DISABLED)
        
        bNextTrial = Button(fChoice, text="Next trial", width=14, command=nextTrial, state=DISABLED)

        rbXisRef.grid(sticky=W)
        rbXisChal.grid(sticky=W)
        bNextTrial.grid()

        fChoice.columnconfigure(0, pad=5)
        fChoice.columnconfigure(1, pad=5)
        fChoice.rowconfigure(0, pad=5)
        fChoice.rowconfigure(1, pad=5)


        #####################
        # the chooser frame #
        #####################

        plc = PlaytimeChooser(fChooser, 200, 20)

        bResetStart = Button(fChooser, text="Reset start", command=resetStart)
        bResetStop  = Button(fChooser, text="Reset stop", command=resetStop)

        bResetStart.grid(row=0, column=0)
        plc.can.grid(row=0, column=1)
        bResetStop.grid(row=0, column=2)

        ###################
        # the status frame#
        ###################
        
        status.set("Stopped.")
        score.set("0/0")
        guessed.set("N/A")
        lScoreTxt   = Label(fStatus, text="Score: ")
        lScore      = Label(fStatus, textvariable=score)
        lGuessedTxt = Label(fStatus, text="Probability that you were guessing: ")
        lGuessed    = Label(fStatus, textvariable=guessed)
        lInfo       = Label(fStatus, text="(Info: Probability should be under 1% to be sure)")
        lStatus     = Label(fStatus, textvariable=status, relief=SUNKEN)

        lScoreTxt.grid(row=0, column=0, sticky=E)
        lScore.grid(row=0, column=1)
        lGuessedTxt.grid(row=1, column=0, sticky=E)
        lGuessed.grid(row=1, column=1)
        lInfo.grid(row=2, column=0, columnspan=2)
        lStatus.grid(row=3, column=0, columnspan=2, sticky=W+E)

        #--------------------#
        root.mainloop()
        

#########################
#########################

id = "foo" #id needed for future versions with save/restore

t = Test(id)
t.stop() # make sure, that no track is playing when starting the script

s = Statistics()
s.newTest(t)

Comparator(t, s)
